<?php
namespace MageDeveloper\Magepdf\Service;

class SettingsService
{
	/**
	 * @var mixed
	 */
	protected $settings = null;

	/**
	 * @var mixed
	 */
	protected $configuration = null;
	
	/**
	 * @var \TYPO3\CMS\Extbase\Configuration\ConfigurationManagerInterface
	 * @inject
	 */
	protected $configurationManager;

	/**
	 * Returns all settings.
	 *
	 * @param string $extensionName Extension Key
	 * @param string $pluginName Plugin Name
	 * @return array
	 */
	public function getSettings($extensionName = null, $pluginName = null)
	{
		if ($this->settings === NULL)
		{

			$this->settings = $this->configurationManager->getConfiguration(
				\TYPO3\CMS\Extbase\Configuration\ConfigurationManagerInterface::CONFIGURATION_TYPE_SETTINGS,
				$extensionName,
				$pluginName
			);
		}
		return $this->settings;
	}

	/**
	 * Returns configuration.
	 *
	 * @param string $extensionName Extension Key
	 * @param string $pluginName Plugin Name
	 * @return array
	 */
	public function getFullConfiguration($extensionName = null, $pluginName = null)
	{
		if ($this->configuration === NULL)
		{
			$this->configuration = $this->configurationManager->getConfiguration(
				\TYPO3\CMS\Extbase\Configuration\ConfigurationManagerInterface::CONFIGURATION_TYPE_FULL_TYPOSCRIPT,
				$extensionName,
				$pluginName
			);

		}

		return $this->configuration;
	}

	/**
	 * Sets the configuration
	 *
	 * @param \array $configuration
	 * @return void
	 */
	public function setConfiguration(array $configuration)
	{
		$this->configuration = $configuration;
	}

	/**
	 * Get configuration setting
	 *
	 * @param string $path Path to configuration
	 * @return mixed
	 */
	public function getConfiguration($path)
	{
		$nodes = explode('.', $path);
		$fullConfiguration = $this->getFullConfiguration();

		$config = null;
		$current = 0;
		$depth = count($nodes);
		$config = $fullConfiguration["plugin."]["tx_magepdf."];
		
		while (is_array($config))
		{
			$node = ($current+1 < $depth)?$nodes[$current].'.':$nodes[$current];

			if(array_key_exists($node, $config))
			{
				$config = $config[$node];
				$current++;
			}
			else
			{

				if (array_key_exists($node.'.', $config))
				{
					$config = $config[$node.'.'];
				}
				else
				{
					$config = "";
				}

				break;
			}

		}

		return $config;
	}

	/**
	 * Gets a setting by code
	 *
	 * @param string $code Setting Code
	 * @return mixed
	 */
	public function getSettingByCode($code)
	{
		$settings = $this->getSettings();

		if (array_key_exists($code, $settings))
		{
			return $settings[$code];
		}

		return "";
	}

	/**
	 * Get an value from the extension configuration
	 * 
	 * @param $config Configuration Name
	 * @return mixed
	 */
	public function getExtensionConfiguration($config)
	{
		$modConfigArray = unserialize( $GLOBALS['TYPO3_CONF_VARS']['EXT']['extConf']["magepdf"]);
		
		if ($modConfigArray[$config]) 
		{
			return $modConfigArray[$config];
		}
		
		return false;
	}
	
}