<?php
namespace MageDeveloper\Magepdf\ViewHelpers;

use TYPO3\CMS\Core\Utility\GeneralUtility;

class PdfViewHelper extends AbstractViewHelper
{
	/**
	 * Dompdf Model
	 * @var \MageDeveloper\Magepdf\Domain\Model\Dompdf
	 */
	protected $pdf;

	/**
	 * Constructor
	 */
	public function __construct()
	{
		parent::__construct();
		$this->pdf = $this->objectManager->get("MageDeveloper\\Magepdf\\Domain\\Model\\Dompdf");
	}

	/**
	 * Gets the pdf model
	 * 
	 * @return \MageDeveloper\Magepdf\Domain\Model\Dompdf|null
	 */
	public function getPdf()
	{
		return $this->pdf;
	}

	/**
	 * Sets the pdf model
	 * 
	 * @param \MageDeveloper\Magepdf\Domain\Model\Dompdf $pdf
	 * @return void
	 */
	public function setPdf(\MageDeveloper\Magepdf\Domain\Model\Dompdf $pdf)
	{
		$this->pdf = $pdf;
	}

	/**
	 * Initialize arguments.
	 *
	 * @return void
	 * @api
	 */
	public function initializeArguments()
	{
		parent::initializeArguments();

		$this->registerArgument("outputFilename", "string", "The output filename to use when downloading", false, "output.pdf");
		$this->registerArgument("forceDownload", "bool", "Force PDF to download", false, false);
		$this->registerArgument("basePath", "string", "Sets the base path, used for external stylesheets and images", false, "");
		$this->registerArgument("utf8encode", "bool", "Encode HTML String to UTF-8", false, false);
		$this->registerArgument("utf8decode", "bool", "Encode HTML String to UTF-8", false, false);

		// Paper Size & Orientation
		$this->registerArgument("paperSize", "string", "The paper size format", false, "A4");
		$this->registerArgument("orientation", "string", "The paper orientation", false, "portrait");
		$this->registerArgument("variables", "array", "Additional Variables to use", false, array());
		$this->registerArgument("remoteUrlsEnabled", "bool", "Enable Remote URLs", false, false);
		$this->registerArgument("options", "array", "PDF Options", false, array());
	}


	/**
	 * Main Render Method
	 * 
	 * @return void
	 */
	public function render()
	{
		ob_end_clean();

		$forceDownload = false;
		if (array_key_exists("forceDownload", $this->arguments))
		{
			$forceDownload = (bool)$this->arguments["forceDownload"];
		}

		// Initialize PDF
		$this->initPdf();

		$outputHtml = $this->renderChildren();

		// Render damn INT_SCRIPT :P / Generates poor HTML Code from Plugins
		/**************************************************************/
		/* @var \TYPO3\CMS\Frontend\Controller\TypoScriptFrontendController $tsFrontendController */
		if (is_array($GLOBALS['TSFE']->config['INTincScript']))
		{
			$tsFrontendController          = $this->objectManager->get("TYPO3\\CMS\\Frontend\\Controller\\TypoScriptFrontendController", $GLOBALS["TYPO3_CONF_VARS"], $GLOBALS["TSFE"]->id, $GLOBALS["TSFE"]->type);
			$tsFrontendController->content = $outputHtml;
			$tsFrontendController->config  = $GLOBALS["TSFE"]->config;
			$tsFrontendController->INTincScript();
			$outputHtml = $tsFrontendController->content;
		}
		/**************************************************************/
		

		// UTF-8 Decode
		$outputHtml = ($this->arguments["utf8decode"])?utf8_decode($outputHtml):$outputHtml;

		// UTF-8 Encode
		$outputHtml = ($this->arguments["utf8encode"])?utf8_encode($outputHtml):$outputHtml;		
		
		$this->renderPdfFromHtml($outputHtml, $forceDownload);

		exit();
	}

	/**
	 * Initialize PDF
	 * 
	 * @return void
	 */
	public function initPdf()
	{
		// Set Paper Size
		$this->getPdf()->setPaperSize( $this->arguments["paperSize"], $this->arguments["orientation"] );

		// Base Path
		$path = GeneralUtility::getFileAbsFileName($this->arguments["basePath"]);
		$this->getPdf()->setBasePath( $path );

		$this->templateVariableContainer->add("basePath", $path);
		$this->templateVariableContainer->add("paperSize", $this->arguments["paperSize"]);
		$this->templateVariableContainer->add("orientation", $this->arguments["orientation"]);
		$this->templateVariableContainer->add("outputFilename", $this->arguments["outputFilename"]);
		

		// Add additional variables to the container
		$variables = $this->arguments["variables"];
		if (is_array($variables) && !empty($variables))
		{
			foreach($variables as $_varName=>$_varVal)
			{
				$this->templateVariableContainer->add($_varName, $_varVal);
			}
		}
		
		// Options
		$options = $this->arguments["options"];
		if (is_array($options) && !empty($options))
		{
			foreach($options as $_opt=>$_optVal)
			{
				$this->getPdf()->setOption($_opt, $_optVal);
			}
		}			
		
		if ($this->arguments["remoteUrlsEnabled"])
		{
			$this->getPdf()->setOption("enable_remote", true);
		}
		
		
		
		return;
	}

	/**
	 * Renders an pdf file from html code
	 * 
	 * @param string $html HTML Code to render
	 * @param bool $forceDownload Force Download of PDF
	 * @return void
	 */
	public function renderPdfFromHtml($html, $forceDownload = false)
	{
		// Inject Html
		$this->getPdf()->setHtml( $html );
		
		$this->getPdf()->render();
		
		// Stream Context
		$this->getPdf()->setStream( $this->arguments["outputFilename"], array("Attachment" => $forceDownload) );
	}


}